#ifdef __IPHONEOS__
#include "ios.h"
#endif

#include "utils.h"
#include "graphics.h"
#include "sound.h"
#include "game.h"
#include "themes.h"
#include "menu.h"
#include "config.h"
#include "input.h"
#include "instead/instead.h"
