#ifndef _IOS_H_INCLUDED
#define _IOS_H_INCLUDED
#define VERSION "3.5.2"
#define STEAD_PATH "__private__/stead"
#define THEMES_PATH "__private__/themes"
#define GAMES_PATH "__private__/games"
#define ICON_PATH "__private__/icon"
#define LANG_PATH "__private__/lang"
extern int setup_inbox(void);
extern void ios_cfg_init(void);
#endif
